var gulp = require("gulp");
var fileUrl = require("file-url");
var path = require("path");
var xtend = require("xtend");
var rfr = require("rfr");

var jade = require("gulp-jade");
var babel = require("gulp-babel");
var livereload = require("gulp-livereload");
var cache = require("gulp-cached");
var remember = require("gulp-remember");
var plumber = require("gulp-plumber");
var rename = require("gulp-rename");
var namedLog = require("gulp-named-log");

var runProcess = rfr("lib/gulp/run-process");
var patchLivereloadLogger = rfr("lib/gulp/patch-livereload-logger");

var babelLogger = namedLog("babel");
var jadeLogger = namedLog("jade");
var electronLogger = namedLog("electron");
var livereloadLogger = namedLog("livereload");

patchLivereloadLogger(livereload, livereloadLogger);

var sources = {
	"babel-main": "app.js",
	"babel-lib": "src/**/*.js",
	"jade-views": "src/views/**/*.jade"
}

function getLiveReloadStream() {
	var lr = livereload({
		quiet: true
	});

	return lr;
}

gulp.task("electron", function() {
	var electronProcess = runProcess(path.join(__dirname, "node_modules/.bin/electron"), [__dirname], {
		logger: electronLogger,
		env: xtend(process.env, {
			NODE_ENV: "development"
		})
	});
});

gulp.task('babel-main', function() {
	return gulp.src(sources["babel-main"])
		.pipe(plumber())
		.pipe(cache("babel-main"))
		.pipe(babel({presets: ["es2015"]}).on('error', babelLogger.error)).on('data', babelLogger.log)
		.pipe(getLiveReloadStream())
		.pipe(remember("babel-main"))
		.pipe(rename("app.es5.js"))
		.pipe(gulp.dest("./"));
});

gulp.task('babel-lib', function() {
	return gulp.src(sources["babel-lib"])
		.pipe(plumber())
		.pipe(cache("babel-lib"))
		.pipe(babel({presets: ["es2015"]}).on('error', babelLogger.error)).on('data', babelLogger.log)
		.pipe(getLiveReloadStream())
		.pipe(remember("babel-lib"))
		.pipe(gulp.dest("lib/"));
});

gulp.task("jade-views", function() {
	return gulp.src(sources["jade-views"])
		.pipe(plumber())
		.pipe(cache("jade-views"))
		.pipe(jade().on('error', jadeLogger.error)).on('data', jadeLogger.log)
		.pipe(getLiveReloadStream())
		.pipe(remember("jade-views"))
		.pipe(gulp.dest("lib/views/"));
});

gulp.task('watch', function () {
	livereload.listen();

	gulp.watch(sources["babel-lib"], ["babel-lib"]);
	gulp.watch(sources["babel-main"], ["babel-main"]);
	gulp.watch(sources["jade-views"], ["jade-views"]);
});

gulp.task('default', ['babel-lib', 'babel-main', 'jade-views', 'watch', 'electron']);
